/*
 * @(#)StyleTableDemo.java 6/17/2005
 *
 * Copyright 2002 - 2005 JIDE Software Inc. All rights reserved.
 */

import com.jidesoft.dialog.ButtonPanel;
import com.jidesoft.grid.AbstractStyleTableModel;
import com.jidesoft.grid.CellStyle;
import com.jidesoft.grid.CellStyleTable;
import com.jidesoft.grid.JideTable;
import com.jidesoft.plaf.LookAndFeelFactory;
import com.jidesoft.utils.SystemInfo;

import javax.swing.*;
import javax.swing.table.AbstractTableModel;
import java.awt.*;
import java.awt.event.ActionEvent;

/**
 * Demoed Component: {@link com.jidesoft.grid.CellStyleTable}
 * <br>
 * Required jar files: jide-common.jar, jide-grids.jar
 * <br>
 * Required L&F: any L&F
 */
public class AbstractStyleTableDemo extends AbstractDemo {
    public JideTable _table;
    protected AbstractTableModel _tableModel;
    protected int _pattern;

    public AbstractStyleTableDemo() {
    }

    public String getName() {
        return "CellStyleTable Demo (Predefined Patterns)";
    }

    public String getProduct() {
        return PRODUCT_NAME_GRIDS;
    }

    public String getDescription() {
        return "This is a demo of CellStyleTable.\n" +
                "\n" +
                "Demoed classes:\n" +
                "com.jidesoft.grid.DefaultStyleTableModel\n" +
                "com.jidesoft.grid.CellStyleTable";
    }

    class StyleTableTableModel extends AbstractStyleTableModel {
        protected final Color BACKGROUND1 = new Color(253, 253, 244);
        protected final Color BACKGROUND2 = new Color(230, 230, 255);
        protected final Color BACKGROUND3 = new Color(210, 255, 210);

        public int getRowCount() {
            return 10000;
        }

        public int getColumnCount() {
            return 9;
        }

        public boolean isCellEditable(int rowIndex, int columnIndex) {
            return false;
        }

        public Object getValueAt(int rowIndex, int columnIndex) {
            return "" + rowIndex + "," + columnIndex;
        }

        CellStyle _cellStyle = new CellStyle();

        public CellStyle getCellStyleAt(int row, int column) {
            _cellStyle.setHorizontalAlignment(SwingConstants.CENTER);
            if (_pattern == 0) {
                if (row % 2 == 0) {
                    _cellStyle.setBackground(BACKGROUND1);
                }
                else {
                    _cellStyle.setBackground(BACKGROUND2);
                }
            }
            else if (_pattern == 1) {
                if (row % 3 == 0) {
                    _cellStyle.setBackground(BACKGROUND2);
                }
                else if (row % 3 == 1) {
                    _cellStyle.setBackground(BACKGROUND1);
                }
                else {
                    _cellStyle.setBackground(BACKGROUND3);
                }
            }
            else if (_pattern == 2) {
                if (column % 2 == 0) {
                    _cellStyle.setBackground(BACKGROUND1);
                }
                else {
                    _cellStyle.setBackground(BACKGROUND2);
                }
            }
            else {
                if (row % 2 == column % 2) {
                    _cellStyle.setBackground(BACKGROUND1);
                }
                else {
                    _cellStyle.setBackground(BACKGROUND2);
                }
            }
            return _cellStyle;
        }

        public boolean isCellStyleOn() {
            return true;
        }
    }

    public Component getDemoPanel() {
        JPanel panel = new JPanel(new BorderLayout(6, 6));
        _tableModel = new StyleTableTableModel();
        _table = new CellStyleTable(_tableModel);
        _table.setRowSelectionAllowed(true);
        _table.setColumnSelectionAllowed(true);
        _table.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
        _table.getColumnModel().getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);

        panel.add(new JScrollPane(_table), BorderLayout.CENTER);
        return panel;
    }

    public Component getOptionsPanel() {
        ButtonPanel panel = new ButtonPanel(SwingConstants.TOP);
        JRadioButton rowStripe = new JRadioButton(new AbstractAction("Row Stripe (two colors)") {
            public void actionPerformed(ActionEvent e) {
                _pattern = 0;
                _table.repaint();
            }
        });
        panel.addButton(rowStripe);
        JRadioButton row3Stripe = new JRadioButton(new AbstractAction("Row Stripe (three colors)") {
            public void actionPerformed(ActionEvent e) {
                _pattern = 1;
                _table.repaint();
            }
        });
        panel.addButton(row3Stripe);
        JRadioButton columnStripe = new JRadioButton(new AbstractAction("Column Stripe") {
            public void actionPerformed(ActionEvent e) {
                _pattern = 2;
                _table.repaint();
            }
        });
        panel.addButton(columnStripe);
        JRadioButton grid = new JRadioButton(new AbstractAction("Grid") {
            public void actionPerformed(ActionEvent e) {
                _pattern = 3;
                _table.repaint();
            }
        });
        rowStripe.setSelected(_pattern == 0);
        row3Stripe.setSelected(_pattern == 1);
        columnStripe.setSelected(_pattern == 2);
        grid.setSelected(_pattern == 3);
        panel.addButton(grid);
        ButtonGroup buttonGroup = new ButtonGroup();
        buttonGroup.add(rowStripe);
        buttonGroup.add(row3Stripe);
        buttonGroup.add(columnStripe);
        buttonGroup.add(grid);
        return panel;
    }

    public String getDemoFolder() {
        return "G17. StyleTable";
    }

    public static void main(String[] args) {
        LookAndFeelFactory.installDefaultLookAndFeelAndExtension();
        showAsFrame(new AbstractStyleTableDemo());
    }
}

