/*
 * @(#)ComboBoxDemo.java 2/14/2005
 *
 * Copyright 2002 - 2005 JIDE Software Inc. All rights reserved.
 */

import com.jidesoft.combobox.*;
import com.jidesoft.plaf.LookAndFeelFactory;
import com.jidesoft.swing.JideBoxLayout;

import javax.swing.*;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import java.awt.*;
import java.text.DateFormat;
import java.util.Calendar;

/**
 * Demoed Component: {@link ListComboBox}, {@link FileChooserComboBox}, {@link ColorComboBox}, {@link DateComboBox}
 * <br>
 * Required jar files: jide-common.jar, jide-grids.jar
 * <br>
 * Required L&F: Jide L&F extension required
 */
public class ComboBoxDemo extends AbstractDemo {

    public ComboBoxDemo() {
    }

    public String getName() {
        return "ComboBox Demo";
    }

    public int getAttributes() {
        return ATTRIBUTE_UPDATED;
    }

    public String getProduct() {
        return PRODUCT_NAME_GRIDS;
    }

    public String getDescription() {
        return "This is a demo of various comboboxes. \n" +
                "\n" +
                "Demoed classes:\n" +
                "com.jidesoft.combobox.DateComboBox\n" +
                "com.jidesoft.combobox.ColorComboBox\n" +
                "com.jidesoft.combobox.TreeComboBox\n" +
                "com.jidesoft.combobox.ListComboBox\n" +
                "com.jidesoft.combobox.FileChooserComboBox";
    }

    public Component getDemoPanel() {
        JPanel panel = new JPanel();
        panel.setLayout(new JideBoxLayout(panel, JideBoxLayout.Y_AXIS));
        panel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

        JComboBox comboBox = new JComboBox(new String[]{"True", "False"});
        comboBox.setEditable(true);
        comboBox.setToolTipText("Regular JComboBox for comparision");
        panel.add(new JLabel("Regular JComboBox"));
        panel.add(Box.createVerticalStrut(3), JideBoxLayout.FIX);
        panel.add(comboBox);
        panel.add(Box.createVerticalStrut(12), JideBoxLayout.FIX);

        // create boolean combobox
        AbstractComboBox booleanComboBox = createBooleanComboBox();
        booleanComboBox.setToolTipText("ComboBox to choose boolean");
        booleanComboBox.setSelectedItem(Boolean.FALSE);
        booleanComboBox.setPrototypeDisplayValue(Boolean.FALSE);
        panel.add(new JLabel("ListComboBox (Boolean)"));
        panel.add(Box.createVerticalStrut(3), JideBoxLayout.FIX);
        panel.add(booleanComboBox);
        panel.add(Box.createVerticalStrut(12), JideBoxLayout.FIX);

        // create font name combobox
        AbstractComboBox fontNameComboBox = createFontNameComboBox();
        fontNameComboBox.setToolTipText("ComboBox to choose font name from a list");
        fontNameComboBox.setEditable(false);
        fontNameComboBox.setSelectedItem("Arial");

        panel.add(new JLabel("ListComboBox (Font Name) (Non-editable)"));
        panel.add(Box.createVerticalStrut(3), JideBoxLayout.FIX);
        panel.add(fontNameComboBox);
        panel.add(Box.createVerticalStrut(12), JideBoxLayout.FIX);

        // create font name combobox
        AbstractComboBox fontNameComboBox2 = createFontNameComboBox();
        fontNameComboBox2.setToolTipText("ComboBox to choose font name or type in font name directly");
        fontNameComboBox2.setSelectedItem("Arial");
        panel.add(new JLabel("ListComboBox (Font Name) (Editable)"));
        panel.add(Box.createVerticalStrut(3), JideBoxLayout.FIX);
        panel.add(fontNameComboBox2);
        panel.add(Box.createVerticalStrut(12), JideBoxLayout.FIX);

        // create tree combobox
        AbstractComboBox treeComboBox = new TreeComboBox() {
            protected boolean isValidSelection(TreePath path) {
                TreeNode treeNode = (TreeNode) path.getLastPathComponent();
                return treeNode.isLeaf();
            }
        };
        treeComboBox.setEditable(false);
        treeComboBox.setToolTipText("ComboBox which has a tree as selection list");
        panel.add(new JLabel("TreeComboBox (Non-editable, only leaf node selectable)"));
        panel.add(Box.createVerticalStrut(3), JideBoxLayout.FIX);
        panel.add(treeComboBox);
        panel.add(Box.createVerticalStrut(12), JideBoxLayout.FIX);

        // create tree combobox
        AbstractComboBox treeComboBox2 = new TreeComboBox();
        treeComboBox2.setToolTipText("ComboBox which has a tree as selection list");
        panel.add(new JLabel("TreeComboBox (Editable)"));
        panel.add(Box.createVerticalStrut(3), JideBoxLayout.FIX);
        panel.add(treeComboBox2);
        panel.add(Box.createVerticalStrut(12), JideBoxLayout.FIX);

//        // create date combobox
        DateComboBox dateComboBox = new DateComboBox();
//        dateComboBox.setDate(Calendar.getInstance().getTime());
        dateComboBox.setToolTipText("ComboBox to chooose date");
        dateComboBox.setDate(null);

        panel.add(new JLabel("DateComboBox"));
        panel.add(Box.createVerticalStrut(3), JideBoxLayout.FIX);
        panel.add(dateComboBox);
        panel.add(Box.createVerticalStrut(12), JideBoxLayout.FIX);

        // create date combobox
        DateComboBox datetimeComboBox = new DateComboBox();
        datetimeComboBox.setTimeDisplayed(true);
        datetimeComboBox.setFormat(DateFormat.getDateTimeInstance());
        datetimeComboBox.setTimeFormat("hh:mm:ss a");
        datetimeComboBox.setDate(Calendar.getInstance().getTime());
        datetimeComboBox.setToolTipText("ComboBox to chooose date and time");

        panel.add(new JLabel("DateComboBox (Time Displayed)"));
        panel.add(Box.createVerticalStrut(3), JideBoxLayout.FIX);
        panel.add(datetimeComboBox);
        panel.add(Box.createVerticalStrut(12), JideBoxLayout.FIX);

        // create color combobox
        ColorComboBox colorComboBox = new ColorComboBox();
        colorComboBox.setSelectedColor(Color.GREEN);
        colorComboBox.setToolTipText("ComboBox to chooose color");
        panel.add(new JLabel("ColorComboBox"));
        panel.add(Box.createVerticalStrut(3), JideBoxLayout.FIX);
        panel.add(colorComboBox);
        panel.add(Box.createVerticalStrut(12), JideBoxLayout.FIX);

        // create font combobox
        FontComboBox fontComboBox = new FontComboBox();
        fontComboBox.setToolTipText("ComboBox to choose a font");
        JLabel label = new JLabel("FontComboBox");
        panel.add(label);
        fontComboBox.setSelectedFont(label.getFont());
        panel.add(Box.createVerticalStrut(3), JideBoxLayout.FIX);
        panel.add(fontComboBox);
        panel.add(Box.createVerticalStrut(12), JideBoxLayout.FIX);

        // create file chooser combobox
        FileChooserComboBox fileComboBox = new FileChooserComboBox();
        fileComboBox.setToolTipText("ComboBox to choose a file");
        panel.add(new JLabel("FileChooserComboBox"));
        panel.add(Box.createVerticalStrut(3), JideBoxLayout.FIX);
        panel.add(fileComboBox);
        panel.add(Box.createVerticalStrut(12), JideBoxLayout.FIX);

        // create font combobox
        InsetsComboBox insetsComboBox = new InsetsComboBox();
        insetsComboBox.setToolTipText("ComboBox to choose an insets");
        panel.add(new JLabel("InsetsComboBox"));
        panel.add(Box.createVerticalStrut(3), JideBoxLayout.FIX);
        panel.add(insetsComboBox);
        panel.add(Box.createVerticalStrut(12), JideBoxLayout.FIX);

        panel.add(Box.createGlue(), JideBoxLayout.VARY);

        return panel;
    }

    public String getDemoFolder() {
        return "G4. ComboBoxes";
    }

    public static void main(String[] args) {
        LookAndFeelFactory.installDefaultLookAndFeelAndExtension();
        showAsFrame(new ComboBoxDemo());
    }

    private static ListComboBox createBooleanComboBox() {
        ListComboBox booleanComboBox = new ListComboBox(ListComboBox.BOOLEAN_ARRAY, Boolean.class);
        return booleanComboBox;
    }

    private static ListComboBox createFontNameComboBox() {
        String[] fontNames = GraphicsEnvironment.getLocalGraphicsEnvironment().getAvailableFontFamilyNames();
        ListComboBox fontNameComboBox = new ListComboBox(fontNames, String.class);
        return fontNameComboBox;
    }
}
