/*
 * @(#)FilterableTableDemo.java
 *
 * Copyright 2002 - 2003 JIDE Software Inc. All rights reserved.
 */

import com.jidesoft.grid.*;
import com.jidesoft.plaf.LookAndFeelFactory;
import com.jidesoft.swing.JidePopupMenu;
import com.jidesoft.swing.JideTabbedPane;

import javax.swing.*;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableModel;
import java.awt.*;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

/**
 * Demoed Component: {@link FilterableTableModel}
 * <br>
 * Required jar files: jide-common.jar, jide-grids.jar
 * <br>
 * Required L&F: any L&F
 */
public class FilterableTableDemo extends AbstractDemo {
    public JCheckBox _booleanFilterCheckBox;
    public JCheckBox _intFilterCheckBox;
    private FilterableTableModel _filterableTableModel;
    private AbstractFilter _booleanFilter;
    private AbstractFilter _intFilter;

    public FilterableTableDemo() {
    }

    public String getName() {
        return "FilterableTableModel Demo";
    }

    public String getProduct() {
        return PRODUCT_NAME_GRIDS;
    }

    public String getDescription() {
        return "This is a demo of FilterableTableModel. FilterableTableModel is a table model which supports filters on columns. You can add filters so that only data the satisfied the filters will be shown in the table.\n" +
                "\n" +
                "Demoed classes:\n" +
                "com.jidesoft.grid.FilterableTableModel\n" +
                "com.jidesoft.grid.Filter";
    }

    public Component getOptionsPanel() {
        JPanel checkBoxPanel = new JPanel(new GridLayout(2, 1));
        _booleanFilterCheckBox = new JCheckBox("Filter " + _booleanFilter.getName());
        _booleanFilterCheckBox.setSelected(true);
        _booleanFilterCheckBox.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                _booleanFilter.setEnabled(_booleanFilterCheckBox.isSelected());
                _filterableTableModel.refresh();
            }
        });
        _booleanFilter.addFilterListener(new FilterListener() {
            public void filterChanged(FilterEvent event) {
                if (event.getID() == FilterEvent.FILTER_ENABLED) {
                    _booleanFilterCheckBox.setSelected(true);
                }
                else if (event.getID() == FilterEvent.FILTER_DISABLED) {
                    _booleanFilterCheckBox.setSelected(false);
                }
            }
        });
        _intFilterCheckBox = new JCheckBox("Filter " + _intFilter.getName());
        _intFilterCheckBox.setSelected(true);
        _intFilterCheckBox.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                _intFilter.setEnabled(_intFilterCheckBox.isSelected());
                _filterableTableModel.refresh();
            }
        });
        _intFilter.addFilterListener(new FilterListener() {
            public void filterChanged(FilterEvent event) {
                if (event.getID() == FilterEvent.FILTER_ENABLED) {
                    _intFilterCheckBox.setSelected(true);
                }
                else if (event.getID() == FilterEvent.FILTER_DISABLED) {
                    _intFilterCheckBox.setSelected(false);
                }
            }
        });
        checkBoxPanel.add(_intFilterCheckBox);
        checkBoxPanel.add(_booleanFilterCheckBox);
        return checkBoxPanel;
    }

    public Component getDemoPanel() {
        initFilters();

        JideTabbedPane tabbedPane = new JideTabbedPane();
        tabbedPane.setBoxStyleTab(true);

        TableModel model = new SampleTableModel();

        _filterableTableModel = new FilterableTableModel(model);
        _filterableTableModel.addFilter(0, _intFilter);
        _filterableTableModel.addFilter(2, _booleanFilter);
        _filterableTableModel.setFiltersApplied(true);

        final SortableTable sortableTable = new SortableTable(_filterableTableModel);
        sortableTable.setColumnSelectionAllowed(true);
        sortableTable.setRowSelectionAllowed(true);

        sortableTable.getTableHeader().addMouseListener(new MouseAdapter() {
            public void mousePressed(MouseEvent e) {
                super.mousePressed(e);
                if (e.isPopupTrigger()) {
                    showPopup(e);
                }
            }

            public void mouseReleased(MouseEvent e) {
                super.mouseReleased(e);
                if (e.isPopupTrigger()) {
                    showPopup(e);
                }
            }

            private void showPopup(MouseEvent e) {
                int column = ((JTableHeader) e.getSource()).getColumnModel().getColumnIndexAtX(e.getPoint().x);
                JMenuItem[] menuItems = _filterableTableModel.getPopupMenuItems(column);
                JPopupMenu popupMenu = new JidePopupMenu();
                popupMenu.setRequestFocusEnabled(false);
                for (int i = 0; i < menuItems.length; i++) {
                    JMenuItem item = menuItems[i];
                    popupMenu.add(item);
                }
                if (popupMenu.getComponentCount() > 0) {
                    popupMenu.show((Component) e.getSource(), e.getPoint().x, e.getPoint().y);
                }
            }
        });

        tabbedPane.addTab("SortableTable", new JScrollPane(sortableTable));
        tabbedPane.addTab("JTable", new JScrollPane(new JTable(_filterableTableModel)));

        return tabbedPane;
    }

    private void initFilters() {
        _intFilter = new AbstractTableFilter("\"int column\" value == 2") {
            public boolean isValueFiltered(Object value) {
//                System.out.println("int " + getRowIndex() + " " + getColumnIndex() + " " + value);
                if (value instanceof Integer) {
                    if (((Integer) value).intValue() == 2)
                        return false;
                }
                return true;
            }
        };

        _booleanFilter = new AbstractTableFilter("\"boolean column\" value == true") {
            public boolean isValueFiltered(Object value) {
//                System.out.println("boolean " + getRowIndex() + " " + getColumnIndex() + " " + value);
                if (Boolean.TRUE.equals(value))
                    return false;
                else
                    return true;
            }
        };
    }

    public static class SampleTableModel extends AbstractTableModel {

        public int getColumnCount() {
            return 4;
        }

        public int getRowCount() {
            return 8;
        }

        public String getColumnName(int column) {
            switch (column) {
                case 0:
                    return "int column";
                case 1:
                    return "double column";
                case 2:
                    return "boolean column";
                case 3:
                    return "string column";
            }
            return "";
        }

        public Class getColumnClass(int column) {
            switch (column) {
                case 0:
                    return Integer.class;
                case 1:
                    return Double.class;
                case 2:
                    return Boolean.class;
                case 3:
                    return String.class;
            }
            return Object.class;
        }

        public Object getValueAt(int row, int column) {
            switch (column) {
                case 0:
                    if (row > 4) {
                        return new Integer(2);
                    }
                    else {
                        return new Integer(row);
                    }
                case 1:
                    return new Double(row);
                case 2:
                    if (row % 2 == 0)
                        return Boolean.TRUE;
                    return Boolean.FALSE;
                case 3:
                    return new String("row " + (getRowCount() - row));
            }
            return null;
        }
    }

    public String getDemoFolder() {
        return "G7. FilterableTable";
    }

    static public void main(String[] s) {
        LookAndFeelFactory.installDefaultLookAndFeelAndExtension();
        showAsFrame(new FilterableTableDemo());
    }
}
