/*
 * @(#)JideTabbedPaneDemo.java
 *
 * Copyright 2002 JIDE Software Inc. All rights reserved.
 */

import com.jidesoft.icons.JideIconsFactory;
import com.jidesoft.plaf.LookAndFeelFactory;
import com.jidesoft.swing.JideBoxLayout;
import com.jidesoft.swing.JideTabbedPane;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

/**
 * Demoed Component: JideTabbedPane
 * <br>
 * Required jar files: jide-common.jar
 * <br>
 * Required L&F: Jide L&F extension required
 */
public class JideTabbedPaneDemo extends AbstractDemo {

    private static JideTabbedPane _tabbedPane;

    public JideTabbedPaneDemo() {
    }

    public String getName() {
        return "JideTabbedPane Demo";
    }

    public String getProduct() {
        return PRODUCT_NAME_COMPONENTS;
    }

    public String getDescription() {
        return "JideTabbedPane is an extended version JTabbedPane. The additional features are \n" +
                "1. Has an option to hide the tab area if there is only one component in a tabbed pane\n" +
                "2. Has an option to shrink the tab width so that all tabs can always fit in one row\n" +
                "3. Has an option to show scroll left and scroll right buttons and a close button in the right corner of the tab area\n" +
                "4. Has an option to show a close button on tab\n" +
                "5. Supports in-placing tab editing. Mouse double clicks on tab to start editing\n" +
                "6. Supports various LookAndFeels\n" +
                "\n" +
                "Demoed classes:\n" +
                "com.jidesoft.swing.JideTabbedPane\n";
    }

    public Component getDemoPanel() {
        _tabbedPane = createTabbedPane();
        _tabbedPane.setPreferredSize(new Dimension(500, 400));
        return _tabbedPane;
    }

    public String getDemoFolder() {
        return "B6. JideTabbedPane";
    }

    public static void main(String[] args) {
        LookAndFeelFactory.installDefaultLookAndFeelAndExtension();
        showAsFrame(new JideTabbedPaneDemo());
    }

    public Component getOptionsPanel() {
        JPanel panel = new JPanel();
        panel.setLayout(new JideBoxLayout(panel, BoxLayout.Y_AXIS));

        final ButtonGroup group = new ButtonGroup();
        final JRadioButton top = new JRadioButton("Tabs on Top");
        final JRadioButton bottom = new JRadioButton("Tabs on Bottom");
//        final JRadioButton left = new JRadioButton("Tabs on Left");
//        final JRadioButton right = new JRadioButton("Tabs on Right");
        group.add(top);
        group.add(bottom);
//        group.add(left);
//        group.add(right);
        bottom.setSelected(true);
        ItemListener itemListener = new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                if (top.isSelected()) {
                    _tabbedPane.setTabPlacement(JideTabbedPane.TOP);
                }
                else if (bottom.isSelected()) {
                    _tabbedPane.setTabPlacement(JideTabbedPane.BOTTOM);
                }
//                else if (left.isSelected()) {
//                    _tabbedPane.setTabPlacement(JideTabbedPane.LEFT);
//                }
//                else if (right.isSelected()) {
//                    _tabbedPane.setTabPlacement(JideTabbedPane.RIGHT);
//                }
            }
        };
        top.addItemListener(itemListener);
        bottom.addItemListener(itemListener);
//        left.addItemListener(itemListener);
//        right.addItemListener(itemListener);
        panel.add(top);
        panel.add(bottom);
//        panel.add(left);
//        panel.add(right);
        panel.add(Box.createVerticalStrut(20), JideBoxLayout.FIX);
        final JCheckBox show = new JCheckBox("Always Show Buttons");
        show.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                _tabbedPane.setShowTabButtons(show.isSelected());
            }
        });
        panel.add(show);
        final JCheckBox hide = new JCheckBox("Hidden Tab Area if Only One Tab");
        hide.setSelected(_tabbedPane.isHideOneTab());
        hide.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                _tabbedPane.setHideOneTab(hide.isSelected());
            }
        });
        panel.add(hide);
        final JCheckBox showTab = new JCheckBox("Show Tab Area");
        showTab.setSelected(_tabbedPane.isShowTabArea());
        showTab.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                _tabbedPane.setShowTabArea(showTab.isSelected());
            }
        });
        panel.add(showTab);
        final JCheckBox fit = new JCheckBox("Shrink Tabs to Fit in");
        fit.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                _tabbedPane.setShrinkTabs(fit.isSelected());
            }
        });
        panel.add(fit);
        final JCheckBox box = new JCheckBox("Box Style Tabs");
        box.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                _tabbedPane.setBoxStyleTab(box.isSelected());
            }
        });
        panel.add(box);

        final JCheckBox useDefaultIcon = new JCheckBox("Use Default Value of Show Icons");
        panel.add(useDefaultIcon);

        final JCheckBox icon = new JCheckBox("Show Icons");
        icon.setSelected(_tabbedPane.isShowIconsOnTab());
        icon.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                _tabbedPane.setShowIconsOnTab(icon.isSelected());
            }
        });
        panel.add(icon);

        final JCheckBox showCloseButton = new JCheckBox("Show Close Button");
        showCloseButton.setSelected(_tabbedPane.isShowCloseButton());
        showCloseButton.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                _tabbedPane.setShowCloseButton(showCloseButton.isSelected());
            }
        });
        panel.add(showCloseButton);

        useDefaultIcon.setSelected(_tabbedPane.isUseDefaultShowIconsOnTab());
        icon.setEnabled(!useDefaultIcon.isSelected());
        useDefaultIcon.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                _tabbedPane.setUseDefaultShowIconsOnTab(useDefaultIcon.isSelected());
                icon.setEnabled(!useDefaultIcon.isSelected());
            }
        });

        final JCheckBox useDefaultClose = new JCheckBox("Use Default Value of Show Close Button On Tab");
        panel.add(useDefaultClose);

        final JCheckBox close = new JCheckBox("Show Close Button on Tab");
        close.setSelected(_tabbedPane.isShowCloseButtonOnTab());
        close.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                _tabbedPane.setShowCloseButtonOnTab(close.isSelected());
            }
        });
        panel.add(close);

        useDefaultClose.setSelected(_tabbedPane.isUseDefaultShowCloseButtonOnTab());
        close.setEnabled(!useDefaultClose.isSelected());
        useDefaultClose.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                _tabbedPane.setUseDefaultShowCloseButtonOnTab(useDefaultClose.isSelected());
                close.setEnabled(!useDefaultClose.isSelected());
            }
        });

        final JCheckBox toggleFirstTabClosable = new JCheckBox("The First Tab is Closable");
        toggleFirstTabClosable.setSelected(_tabbedPane.isTabClosableAt(0));
        toggleFirstTabClosable.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                _tabbedPane.setTabClosableAt(0, toggleFirstTabClosable.isSelected());
            }
        });
        panel.add(toggleFirstTabClosable);

        final JCheckBox editing = new JCheckBox("Allow Editing Tab Title");
        editing.setSelected(_tabbedPane.isTabEditingAllowed());
        editing.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                _tabbedPane.setTabEditingAllowed(editing.isSelected());
            }
        });
        panel.add(editing);

        return panel;
    }

    private static class TabPanel extends JPanel {
        Icon _icon;
        String _title;
        JComponent _component;

        public TabPanel(String title, Icon icon, JComponent component) {
            _title = title;
            _icon = icon;
            _component = component;
        }

        public Icon getIcon() {
            return _icon;
        }

        public void setIcon(Icon icon) {
            _icon = icon;
        }

        public String getTitle() {
            return _title;
        }

        public void setTitle(String title) {
            _title = title;
        }

        public JComponent getComponent() {
            return _component;
        }

        public void setComponent(JComponent component) {
            _component = component;
        }
    }

    private static JideTabbedPane createTabbedPane() {
        JideTabbedPane tabbedPane = new JideTabbedPane(JideTabbedPane.BOTTOM);
        tabbedPane.setOpaque(true);
        TabPanel panel = createTabPanel(1);
        tabbedPane.addTab(panel.getTitle(), panel.getIcon(), panel.getComponent());
        panel = createTabPanel(2);
        tabbedPane.addTab(panel.getTitle(), panel.getIcon(), panel.getComponent());
        panel = createTabPanel(3);
        tabbedPane.addTab(panel.getTitle(), panel.getIcon(), panel.getComponent());
        panel = createTabPanel(4);
        tabbedPane.addTab(panel.getTitle(), panel.getIcon(), panel.getComponent());
        panel = createTabPanel(5);
        tabbedPane.addTab(panel.getTitle(), panel.getIcon(), panel.getComponent());
        panel = createTabPanel(6);
        tabbedPane.addTab(panel.getTitle(), panel.getIcon(), panel.getComponent());
        panel = createTabPanel(7);
        tabbedPane.addTab(panel.getTitle(), panel.getIcon(), panel.getComponent());
        panel = createTabPanel(8);
        tabbedPane.addTab(panel.getTitle(), panel.getIcon(), panel.getComponent());
        return tabbedPane;
    }

    protected static TabPanel createTabPanel(int index) {
        TabPanel panel = new TabPanel("Tab " + index,
                JideIconsFactory.getImageIcon("jide/dockableframe_" + index + ".gif"),
                new JScrollPane(new JTextArea()));
        return panel;
    }
}
