/*
 * @(#)PopupDemo.java 2/11/2005
 *
 * Copyright 2002 - 2005 JIDE Software Inc. All rights reserved.
 */

import com.jidesoft.docking.DockableFrame;
import com.jidesoft.docking.DockableFramePopup;
import com.jidesoft.docking.DockableHolder;
import com.jidesoft.docking.DockingManager;
import com.jidesoft.plaf.LookAndFeelFactory;
import com.jidesoft.popup.JidePopup;
import com.jidesoft.swing.JideBoxLayout;
import com.jidesoft.swing.SearchableUtils;
import com.jidesoft.utils.SystemInfo;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

/**
 * Demoed Component: {@link com.jidesoft.popup.JidePopup}
 * <br>
 * Required jar files: jide-common.jar
 * <br>
 * Required L&F: Jide L&F extension required
 */
public class PopupDemo extends AbstractDemo {
    public LocationCustomizationPanel _locationPanel;

    public PopupDemo() {
    }

    public String getName() {
        return "PopupDemo";
    }

    public String getProduct() {
        return PRODUCT_NAME_COMPONENTS;
    }

    public String getDescription() {
        return "JidePopup is a popup that is detachable, dragable, resizable, autohide when timeout or clicks outside.\n" +
                "\n" +
                "Demoed classes:\n" +
                "com.jidesoft.popup.JidePopup";
    }


    public Component getDemoPanel() {
        final JPanel panel = new JPanel();
        panel.setLayout(new JideBoxLayout(panel, JideBoxLayout.Y_AXIS));
        panel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

        final JButton attachedButton = new JButton();
        final JidePopup popup = new JidePopup();
        popup.setMovable(true);
        popup.getContentPane().setLayout(new BorderLayout());
        JTextArea view = new JTextArea();
        view.setRows(10);
        view.setColumns(40);
        popup.getContentPane().add(new JScrollPane(view));
        JMenuBar menuBar = new JMenuBar();
        JMenu menu = menuBar.add(new JMenu("File"));
        menu.add("<< Example >>");
        menuBar.add(new JMenu("Edit"));
        menuBar.add(new JMenu("Help"));
        popup.setJMenuBar(menuBar);
        popup.setDefaultFocusComponent(view);
        AbstractAction action = new AbstractAction("Show Attached Popup (regular popup)") {
            public void actionPerformed(ActionEvent e) {
                popup.setOwner(attachedButton);
                popup.setResizable(true);
                popup.setMovable(true);
                if(popup.isPopupVisible()) {
                    popup.hidePopup();
                }
                else {
                    popup.showPopup();
                }
            }
        };
        attachedButton.setAction(action);
        panel.add(attachedButton);
        panel.add(Box.createVerticalStrut(12), JideBoxLayout.FIX);

        if ("window".equalsIgnoreCase(System.getProperty("docking.floatingContainerType"))) {
            final JButton attachedDockableFrameButton = new JButton();
            action = new AbstractAction("Show Attached Popup (dockable frame type popup)") {
                public void actionPerformed(ActionEvent e) {
                    Container topLevelMenuContainer = attachedDockableFrameButton.getTopLevelAncestor();
                    DockingManager dockingManager = null;
                    JidePopup popup = null;
                    if (topLevelMenuContainer instanceof DockableHolder) {
                        dockingManager = ((DockableHolder) topLevelMenuContainer).getDockingManager();
                        popup = new DockableFramePopup(dockingManager) {
                            public DockableFrame createDockableFrame() {
                                return new DockableFrame("Popup", UIManager.getIcon("DockableFrame.defaultIcon"));
                            }
                        };
                    }
                    else {
                        popup = new JidePopup();
                    }

                    popup.setMovable(true);
                    popup.getContentPane().setLayout(new BorderLayout());
                    JTextArea view = new JTextArea();
                    view.setRows(10);
                    view.setColumns(40);
                    popup.getContentPane().add(new JScrollPane(view));
                    JMenuBar menuBar = new JMenuBar();
                    JMenu menu = menuBar.add(new JMenu("File"));
                    menu.add("<< Example >>");
                    menuBar.add(new JMenu("Edit"));
                    menuBar.add(new JMenu("Help"));
                    popup.setJMenuBar(menuBar);
                    popup.setOwner(attachedDockableFrameButton);
                    popup.setResizable(true);
                    popup.setMovable(true);
                    popup.setDefaultFocusComponent(view);
                    popup.showPopup();
                }
            };
            attachedDockableFrameButton.setAction(action);
            panel.add(attachedDockableFrameButton);
            panel.add(Box.createVerticalStrut(12), JideBoxLayout.FIX);
        }

        final JButton detachedButton = new JButton();
        AbstractAction action1 = new AbstractAction("Show Detached Popup") {
            public void actionPerformed(ActionEvent e) {
                popup.setOwner(detachedButton);
                popup.setResizable(true);
                popup.setMovable(true);
                popup.setAttachable(false);
                if(popup.isPopupVisible()) {
                    popup.hidePopup();
                }
                else {
                    popup.showPopup(_locationPanel.getDisplayLocation());
                }
            }
        };
        detachedButton.setAction(action1);
        panel.add(detachedButton);
        panel.add(Box.createVerticalStrut(12), JideBoxLayout.FIX);

        final JButton ideaButton = new JButton();
        action = new AbstractAction("Show IntelliJ IDEA Ctrl-N Popup") {
            public void actionPerformed(ActionEvent e) {
                final JidePopup popup = new JidePopup();
                popup.setMovable(false);
                JPanel panel = new JPanel(new BorderLayout());
                panel.setBorder(BorderFactory.createEmptyBorder(4, 2, 2, 2));
                JComboBox comboBox = new JComboBox(getCountryNames());
                SearchableUtils.installSearchable(comboBox);
                panel.add(new JLabel("<HTML><B>Enter country name:</B></HTML>"), BorderLayout.BEFORE_FIRST_LINE);
                panel.add(comboBox);
                popup.getContentPane().add(panel);
                popup.setOwner(ideaButton.getTopLevelAncestor());
                popup.setResizable(false);
                popup.setMovable(false);
                popup.setDefaultFocusComponent(comboBox);
                popup.setOwner(ideaButton);
                comboBox.registerKeyboardAction(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        popup.hidePopupImmediately();
                    }
                }, KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
                popup.showPopup(_locationPanel.getDisplayLocation());
            }
        };
        ideaButton.setAction(action);
        panel.add(ideaButton);
        panel.add(Box.createVerticalStrut(12), JideBoxLayout.FIX);

        panel.add(Box.createGlue(), JideBoxLayout.VARY);
        return panel;
    }

    public Component getOptionsPanel() {
        _locationPanel = new LocationCustomizationPanel();
        return _locationPanel;
    }

    public String getDemoFolder() {
        return "B9. Popup";
    }

    public static void main(String[] args) {
        LookAndFeelFactory.installDefaultLookAndFeelAndExtension();
        showAsFrame(new PopupDemo());
    }

    public static String[] getCountryNames() {
        return new String[]{
            "Andorra",
            "United Arab Emirates",
            "Afghanistan",
            "Antigua And Barbuda",
            "Anguilla",
            "Albania",
            "Armenia",
            "Netherlands Antilles",
            "Angola",
            "Antarctica",
            "Argentina",
            "American Samoa",
            "Austria",
            "Australia",
            "Aruba",
            "Azerbaijan",
            "Bosnia And Herzegovina",
            "Barbados",
            "Bangladesh",
            "Belgium",
            "Burkina Faso",
            "Bulgaria",
            "Bahrain",
            "Burundi",
            "Benin",
            "Bermuda",
            "Brunei Darussalam",
            "Bolivia",
            "Brazil",
            "Bahamas",
            "Bhutan",
            "Bouvet Island",
            "Botswana",
            "Belarus",
            "Belize",
            "Canada",
            "Cocos (Keeling) Islands",
            "Congo, The Democratic Republic Of The",
            "Central African Republic",
            "Congo",
            "Switzerland",
            "Cte D'Ivoire",
            "Cook Islands",
            "Chile",
            "Cameroon",
            "China",
            "Colombia",
            "Costa Rica",
            "Cuba",
            "Cape Verde",
            "Christmas Island",
            "Cyprus",
            "Czech Republic",
            "Germany",
            "Djibouti",
            "Denmark",
            "Dominica",
            "Dominican Republic",
            "Algeria",
            "Ecuador",
            "Estonia",
            "Egypt",
            "Western Sarara",
            "Eritrea",
            "Spain",
            "Ethiopia",
            "Finland",
            "Fiji",
            "Falkland Islands (Malvinas)",
            "Micronesia, Federated States Of",
            "Faroe Islands",
            "France",
            "Gabon",
            "United Kingdom",
            "Grenada",
            "Georgia",
            "French Guiana",
            "Ghana",
            "Gibraltar",
            "Greenland",
            "Gambia",
            "Guinea",
            "Guadeloupe",
            "Equatorial Guinea",
            "Greece",
            "South Georgia And The South Sandwich Islands",
            "Guatemala",
            "Guam",
            "Guinea-bissau",
            "Guyana",
            "Hong Kong",
            "Heard Island And Mcdonald Islands",
            "Honduras",
            "Croatia",
            "Haiti",
            "Hungary",
            "Indonesia",
            "Ireland",
            "Israel",
            "India",
            "British Indian Ocean Territory",
            "Iraq",
            "Iran, Islamic Republic Of",
            "Iceland",
            "Italy",
            "Jamaica",
            "Jordan",
            "Japan",
            "Kenya",
            "Kyrgyzstan",
            "Cambodia",
            "Kiribati",
            "Comoros",
            "Saint Kitts And Nevis",
            "Korea, Democratic People'S Republic Of",
            "Korea, Republic Of",
            "Kuwait",
            "Cayman Islands",
            "Kazakhstan",
            "Lao People'S Democratic Republic",
            "Lebanon",
            "Saint Lucia",
            "Liechtenstein",
            "Sri Lanka",
            "Liberia",
            "Lesotho",
            "Lithuania",
            "Luxembourg",
            "Latvia",
            "Libyan Arab Jamabiriya",
            "Morocco",
            "Monaco",
            "Moldova, Republic Of",
            "Madagascar",
            "Marshall Islands",
            "Macedonia, The Former Yugoslav Repu8lic Of",
            "Mali",
            "Myanmar",
            "Mongolia",
            "Macau",
            "Northern Mariana Islands",
            "Martinique",
            "Mauritania",
            "Montserrat",
            "Malta",
            "Mauritius",
            "Maldives",
            "Malawi",
            "Mexico",
            "Malaysia",
            "Mozambique",
            "Namibia",
            "New Caledonia",
            "Niger",
            "Norfolk Island",
            "Nigeria",
            "Nicaragua",
            "Netherlands",
            "Norway",
            "Nepal",
            "Niue",
            "New Zealand",
            "Oman",
            "Panama",
            "Peru",
            "French Polynesia",
            "Papua New Guinea",
            "Philippines",
            "Pakistan",
            "Poland",
            "Saint Pierre And Miquelon",
            "Pitcairn",
            "Puerto Rico",
            "Portugal",
            "Palau",
            "Paraguay",
            "Qatar",
            "Runion",
            "Romania",
            "Russian Federation",
            "Rwanda",
            "Saudi Arabia",
            "Solomon Islands",
            "Seychelles",
            "Sudan",
            "Sweden",
            "Singapore",
            "Saint Helena",
            "Slovenia",
            "Svalbard And Jan Mayen",
            "Slovakia",
            "Sierra Leone",
            "San Marino",
            "Senegal",
            "Somalia",
            "Suriname",
            "Sao Tome And Principe",
            "El Salvador",
            "Syrian Arab Republic",
            "Swaziland",
            "Turks And Caicos Islands",
            "Chad",
            "French Southern Territories",
            "Togo",
            "Thailand",
            "Tajikistan",
            "Tokelau",
            "Turkmenistan",
            "Tunisia",
            "Tonga",
            "East Timor",
            "Turkey",
            "Trinidad And Tobago",
            "Tuvalu",
            "Taiwan, Province Of China",
            "Tanzania, United Republic Of",
            "Ukraine",
            "Uganda",
            "United States Minor Outlying Islands",
            "United States",
            "Uruguay",
            "Uzbekistan",
            "Venezuela",
            "Virgin Islands, British",
            "Virgin Islands, U.S.",
            "Viet Nam",
            "Vanuatu",
            "Wallis And Futuna",
            "Samoa",
            "Yemen",
            "Mayotte",
            "Yugoslavia",
            "South Africa",
            "Zambia",
            "Zimbabwe"
        };
    }
}

