/*
 * @(#)${NAME}
 *
 * Copyright 2002 - 2004 JIDE Software Inc. All rights reserved.
 */

import com.jidesoft.grid.SortableTable;
import com.jidesoft.grid.TableColumnChooser;
import com.jidesoft.grid.TableUtils;
import com.jidesoft.plaf.LookAndFeelFactory;
import com.jidesoft.utils.Lm;

import javax.swing.*;
import javax.swing.table.AbstractTableModel;
import java.awt.*;
import java.awt.event.ActionEvent;

/**
 * Demoed Component: {@link com.jidesoft.grid.TableUtils}
 * <br>
 * Required jar files: jide-common.jar, jide-grids.jar
 * <br>
 * Required L&F: any L&F
 */
public class TableUtilsDemo extends AbstractDemo {
    private SortableTable _table;

    private int[] _selections;

    private String _tablePref;

    private String _sortingOrders;


    public TableUtilsDemo() {
    }

    public String getName() {
        return "TableUtils Demo";
    }

    public String getProduct() {
        return PRODUCT_NAME_GRIDS;
    }

    public String getDescription() {
        return "This is a demo of TableUtils. TableUtils is a utility class containing several useful methods related to JTable.\n" +
                "\nSave/load selections: saveSelection() method will save the current selection into an int array. You can load the selection back later. This is useful when selection is lost due to table update." +
                "To see a demo, Select several rows in the table, press \"Save Selection\" button, clear the selections by a mouse click on the table, then click on \"Load Selection\" button\n" +
                "\nSave/load table preference: table preference includes table column width and order. After user resized or reordred the table columns, user won't like the information to be persisted so then when table is shown again, it is the same as before. You can use setTablePreference() and getTablePreference() to archive this. " +
                "To see a demo, resize and reorder table columns, press \"Save Preference\" button, resize and reorder more columns, then click on \"Load Preference\" button.\n" +
                "\nThere are several additional methods beginning with \"synchonize...\" which are not covered in this demo. Those methods are for synchonization of several tables such as scrolling, selection, row heights etc. You can find usage of those methods in the demos of TableScrollPane and TableSplitPane.\n" +
                "\n" +
                "Demoed classes:\n" +
                "com.jidesoft.grid.TableUtils";
    }

    public Component getOptionsPanel() {
        JPanel panel = new JPanel(new GridLayout(3, 2));
        JButton saveSelectionButton = new JButton(new AbstractAction("Save Selection") {
            public void actionPerformed(ActionEvent e) {
                _selections = TableUtils.saveSelection(_table);
                Lm.showPopupMessageBox("<HTML>The selection has been saved. You can clear the selection in the table now. " +
                        "<BR>After you clear it, press \"Load Selection\" to restore the saved selection.</HTML>");
            }
        });
        JButton loadSelectionButton = new JButton(new AbstractAction("Load Selection") {
            public void actionPerformed(ActionEvent e) {
                TableUtils.loadSelection(_table, _selections);
            }
        });
        JButton savePrefButton = new JButton(new AbstractAction("Save Preference") {
            public void actionPerformed(ActionEvent e) {
                _tablePref = TableUtils.getTablePreference(_table);
                Lm.showPopupMessageBox("<HTML>The table column width and column order information has been saved. You can change width and order in the table now. " +
                        "<BR>After you are done with it, press \"Load Preference\" to restore the saved column width and order.</HTML>");
            }
        });
        JButton loadPrefButton = new JButton(new AbstractAction("Load Preference") {
            public void actionPerformed(ActionEvent e) {
                TableUtils.setTablePreference(_table, _tablePref);
            }
        });
        JButton saveSortingPrefButton = new JButton(new AbstractAction("Save Sorting Orders") {
            public void actionPerformed(ActionEvent e) {
                _sortingOrders = TableUtils.getSortableTablePreference(_table);
                Lm.showPopupMessageBox("<HTML>The table column width and column order information has been saved. You can change width and order in the table now. " +
                        "<BR>After you are done with it, press \"Load Preference\" to restore the saved column width and order.</HTML>");
            }
        });
        JButton loadSortingPrefButton = new JButton(new AbstractAction("Load Sorting Orders") {
            public void actionPerformed(ActionEvent e) {
                TableUtils.setSortableTablePreference(_table, _sortingOrders);
            }
        });
        panel.add(saveSelectionButton);
        panel.add(loadSelectionButton);
        panel.add(savePrefButton);
        panel.add(loadPrefButton);
        panel.add(saveSortingPrefButton);
        panel.add(loadSortingPrefButton);
        return panel;
    }

    public Component getDemoPanel() {
        _table = new SortableTable(new SampleTableModel());
        TableColumnChooser.install(_table);
        _table.setRowSelectionAllowed(true);
        _table.setColumnSelectionAllowed(true);
        return new JScrollPane(_table);
    }

    public static class SampleTableModel extends AbstractTableModel {

        public int getColumnCount() {
            return 4;
        }

        public int getRowCount() {
            return 8;
        }

        public String getColumnName(int column) {
            switch (column) {
                case 0:
                    return "1: int column";
                case 1:
                    return "2: double column";
                case 2:
                    return "3: boolean column";
                case 3:
                    return "4: string column";
            }
            return "";
        }

        public Class getColumnClass(int column) {
            switch (column) {
                case 0:
                    return Integer.class;
                case 1:
                    return Double.class;
                case 2:
                    return Boolean.class;
                case 3:
                    return String.class;
            }
            return Object.class;
        }

        public Object getValueAt(int row, int column) {
            switch (column) {
                case 0:
                    if (row > 4) {
                        return new Integer(2);
                    }
                    else {
                        return new Integer(row);
                    }
                case 1:
                    return new Double(row);
                case 2:
                    if (row % 2 == 0)
                        return Boolean.TRUE;
                    return Boolean.FALSE;
                case 3:
                    return new String("row " + (getRowCount() - row));
            }
            return null;
        }

    }

    public String getDemoFolder() {
        return "G9. TableUtilsDemo";
    }

    static public void main(String[] s) {
        LookAndFeelFactory.installDefaultLookAndFeelAndExtension();
        showAsFrame(new TableUtilsDemo());
    }
}
