/*
 * @(#)WizardDemo.java
 *
 * Copyright 2002 - 2003 JIDE Software Inc. All rights reserved.
 */

import com.jidesoft.dialog.*;
import com.jidesoft.plaf.LookAndFeelFactory;
import com.jidesoft.swing.JideSwingUtilities;
import com.jidesoft.swing.MultilineLabel;
import com.jidesoft.wizard.*;
import com.jidesoft.utils.SystemInfo;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;
import java.util.List;

/**
 * Demoed Component: {@link WizardDialog}
 * <br>
 * Required jar files: jide-common.jar, jide-components.jar, jide-dialogs.jar
 * <br>
 * Required L&F: Any L&F
 */
public class WizardDemo extends AbstractDemo {
    public String getName() {
        return "Wizard Demo";
    }

    public String getProduct() {
        return PRODUCT_NAME_DIALOGS;
    }

    public String getDescription() {
        return "Wizard is a well-known user interface that is ideal to guide user through for complex and unfamiliar tasks. A typical usage of it is project wizard - which asks user a couple questions and generate source code of a project automatically for user.\n" +
                "There are several wizard standards. The most famous two are Microsoft Wizard 97 standard and Java L&F Wizard standard. Please see references for details. I strongly suggest you read those standards before designing any wizards. Those documents are very well written. They are also the specs for our wizard component. You can find links to them at the reference section of JIDE Dialogs Developer Guide. Our wizard implementation are based on these two standards.\n" +
                "\n" +
                "Demoed classes:\n" +
                "com.jidesoft.wizard.WizardDialog\n" +
                "com.jidesoft.wizard.WelcomeWizardPage\n" +
                "com.jidesoft.wizard.CompletionWizardPage\n" +
                "com.jidesoft.wizard.AbstractWizardPage\n";
    }

    public Component getDemoPanel() {
        final ButtonPanel buttonPanel = new ButtonPanel(SwingConstants.TOP);
        buttonPanel.addButton(new JButton(new AbstractAction("Wizard 97 Style") {
            public void actionPerformed(ActionEvent e) {
                int style = WizardStyle.WIZARD97_STYLE;
                Container topLevelAncestor = buttonPanel.getTopLevelAncestor();
                showWizardDemo(topLevelAncestor instanceof Frame ? (Frame) topLevelAncestor : null, false, style);
            }
        }));

        buttonPanel.addButton(new JButton(new AbstractAction("Java Style") {
            public void actionPerformed(ActionEvent e) {
                int style = WizardStyle.JAVA_STYLE;
                Container topLevelAncestor = buttonPanel.getTopLevelAncestor();
                showWizardDemo(topLevelAncestor instanceof Frame ? (Frame) topLevelAncestor : null, false, style);
            }
        }));
        return buttonPanel;
    }

    public Component getOptionsPanel() {
        return null;
    }

    public String getDemoFolder() {
        return "W1. Wizard";
    }

    public static void main(String[] args) {
// switch follow lines to change style
//        int style = WizardStyle.JAVA_STYLE;
        LookAndFeelFactory.installDefaultLookAndFeelAndExtension();
        showAsFrame(new WizardDemo());
    }

    public static void showWizardDemo(Frame frame, final boolean exit, int style) {
        WizardStyle.setStyle(style);

        final WizardDialog wizard = new WizardDialog(frame, "JIDE Wizard Demo");
        wizard.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        if (style == WizardStyle.WIZARD97_STYLE) {
            wizard.setDefaultGraphic(WizardDemoIconsFactory.getImageIcon(WizardDemoIconsFactory.Wizard97.SAMPLE_IMAGE_SMALL).getImage());
        }
        else {
            wizard.setDefaultGraphic(WizardDemoIconsFactory.getImageIcon(WizardDemoIconsFactory.Metal.SAMPLE_IMAGE_SMALL).getImage());
        }

        wizard.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                super.windowClosing(e);
                if (exit) {
                    System.exit(0);
                }
                else {
                    wizard.dispose();
                }
            }
        });

        // setup model
        PageList model = new PageList();

        AbstractWizardPage page1 = new WelcomePage("Welcome to the JIDE Wizard Feature Demo",
                "This wizard will guide you through the features of Wizard Component from JIDE Software, Inc.");

        AbstractWizardPage page2 = new LicensePage("License Agreement",
                "This page shows you how to enable/disable a button based on user selection.");

        AbstractWizardPage page3 = new InfoWarningPage("Infos and Warnings",
                "This page shows you how to show info message and warning message.");

        AbstractWizardPage page4 = new ChangeNextPagePage("Change Next Page",
                "This page shows you how to change next page based on user selection.");

        AbstractWizardPage page5 = new CompletionPage("Completing the JIDE Wizard Feature Demo",
                "You have successfully run through the important features of Wizard Component from JIDE Software, Inc.");

        model.append(page1);
        model.append(page2);
        model.append(page4);
        model.append(page3);
        model.append(page5);

        wizard.setPageList(model);

        wizard.setFinishAction(new AbstractAction("Finish") {
            public void actionPerformed(ActionEvent e) {
                if (wizard.closeCurrentPage()) {
                    if (exit) {
                        System.exit(0);
                    }
                    else {
                        wizard.dispose();
                    }
                }
            }
        });
        wizard.setCancelAction(new AbstractAction("Cancel") {
            public void actionPerformed(ActionEvent e) {
                if (wizard.closeCurrentPage()) {
                    if (exit) {
                        System.exit(0);
                    }
                    else {
                        wizard.dispose();
                    }
                }
            }
        });

        wizard.pack();
        wizard.setResizable(true); // for wizard, it's better to make it not resizable.
        JideSwingUtilities.globalCenterWindow(wizard);
        wizard.setVisible(true);
    }

    public static class WelcomePage extends WelcomeWizardPage {
        public WelcomePage(String title, String description) {
            super(title, description/*, SampleWizardIconsFactory.getImageIcon(SampleWizardIconsFactory.Wizard97.SAMPLE_IMAGE_SMALL)*/);
        }

        protected void initContentPane() {
            super.initContentPane();
            addText("JIDE Software, Inc. is a leading-edge provider of Swing components for Java developers. JIDE means Java IDE (Integrated Development Environment). As the name indicated, JIDE Software's development components focus on IDE or IDE-like applications for software developers. All our products are in pure Java and Swing, to allow the most compatibilities with industrial standards.");
            addSpace();
            addText("To continue, click Next.");
        }
    }

    public static class CompletionPage extends CompletionWizardPage {
        public CompletionPage(String title, String description) {
            super(title, description);
        }

        protected void initContentPane() {
            super.initContentPane();
            addSpace();
            addText("To close this wizard, click Finish.");
        }
    }

    public static class LicensePage extends AbstractWizardPage {
        private JRadioButton _button1;
        private JRadioButton _button2;

        public LicensePage(String title, String description) {
            super(title, description, WizardDemoIconsFactory.getImageIcon(WizardDemoIconsFactory.Wizard97.SAMPLE_LOGO));
        }

        public int getLeftPaneItems() {
            return (WizardStyle.getStyle() == WizardStyle.JAVA_STYLE) ? LEFTPANE_STEPS | LEFTPANE_HELP : super.getLeftPaneItems();
        }

        public JComponent createWizardContent() {
            JPanel panel = createLicensePanel();
            panel.setBorder(getContentThinBorder());
            return panel;
        }

        public void setupWizardButtons() {
            fireButtonEvent(ButtonEvent.ENABLE_BUTTON, ButtonNames.BACK);
            fireButtonEvent(ButtonEvent.HIDE_BUTTON, ButtonNames.FINISH);
            updateNextButton();
        }

        public JPanel createLicensePanel() {
            JPanel panel = new JPanel(new BorderLayout(4, 4));
            MultilineLabel label = new MultilineLabel("Please read the following license agreement. You must accept the terms of this agreement before continuing the wizard.");
            panel.add(label, BorderLayout.BEFORE_FIRST_LINE);
            panel.add(new JScrollPane(new JTextArea()), BorderLayout.CENTER);
            panel.add(createRadioButtons(), BorderLayout.AFTER_LAST_LINE);
            return panel;
        }

        public JPanel createRadioButtons() {
            _button1 = new JRadioButton("I accept the agreement");
            _button2 = new JRadioButton("I do not accept the agreement");
            _button2.setSelected(true);
            _button1.addItemListener(new ItemListener() {
                public void itemStateChanged(ItemEvent e) {
                    if (e.getStateChange() == ItemEvent.SELECTED) {
                        updateNextButton();
                    }
                }
            });
            _button2.addItemListener(new ItemListener() {
                public void itemStateChanged(ItemEvent e) {
                    if (e.getStateChange() == ItemEvent.SELECTED) {
                        updateNextButton();
                    }
                }
            });
            JPanel panel = new JPanel(new GridLayout(2, 1));
            panel.add(_button1);
            panel.add(_button2);
            ButtonGroup group = new ButtonGroup();
            group.add(_button1);
            group.add(_button2);
            JideSwingUtilities.setOpaqueRecursively(panel, false);
            return panel;
        }

        private void updateNextButton() {
            if (_button1 != null && _button2 != null) {
                if (_button1.isSelected()) {
                    fireButtonEvent(ButtonEvent.ENABLE_BUTTON, ButtonNames.NEXT);
                    fireButtonEvent(ButtonEvent.CHANGE_BUTTON_FOCUS, ButtonNames.NEXT);
                    fireButtonEvent(ButtonEvent.SET_DEFAULT_BUTTON, ButtonNames.NEXT);
                }
                else if (_button2.isSelected()) {
                    fireButtonEvent(ButtonEvent.DISABLE_BUTTON, ButtonNames.NEXT);
                    fireButtonEvent(ButtonEvent.SET_DEFAULT_BUTTON, ButtonNames.BACK);
                }
            }
        }

    }

    public static class InfoWarningPage extends DefaultWizardPage {
        public InfoWarningPage(String title, String description) {
            super(title, description, WizardDemoIconsFactory.getImageIcon(WizardDemoIconsFactory.Wizard97.SAMPLE_LOGO));
        }

        protected void initContentPane() {
            super.initContentPane();
            addInfo("The information bubble icon can emphasize important yet not crucial information. " +
                    "Call addInfo(String text) in DefaultWizardPage to add this type of message.");
            addWarning("The yellow warning icon can emphasize information vital to either the success of the task or the consequences of the actions performed on the wizard page. " +
                    "Call addWarning(String text) in DefaultWizardPage to add this type of message.");
        }

        public void setupWizardButtons() {
            fireButtonEvent(ButtonEvent.ENABLE_BUTTON, ButtonNames.BACK);
            fireButtonEvent(ButtonEvent.ENABLE_BUTTON, ButtonNames.NEXT);
            fireButtonEvent(ButtonEvent.HIDE_BUTTON, ButtonNames.FINISH);
        }
    }

    public static class ChangeNextPagePage extends DefaultWizardPage {
        private JRadioButton _button1;
        private JRadioButton _button2;

        public ChangeNextPagePage(String title, String description) {
            super(title, description, WizardDemoIconsFactory.getImageIcon(WizardDemoIconsFactory.Wizard97.SAMPLE_LOGO));
        }

        public List getSteps() {
            List list = new ArrayList();
            list.add("Welcome to the JIDE Wizard Feature Demo");
            list.add("License Agreement");
            list.add("Change Next Page");
            list.add("...");
            return list;
        }

        public int getSelectedStepIndex() {
            return 2;
        }

        protected void initContentPane() {
            super.initContentPane();
            _button1 = new JRadioButton("If this radio button is selected, press Next button will continue to Infos and Warning page");
            _button2 = new JRadioButton("If this radio button is selected, press Next button will continue to Completion page");
            _button1.addItemListener(new ItemListener() {
                public void itemStateChanged(ItemEvent e) {
                    if (e.getStateChange() == ItemEvent.SELECTED) {
                        updateNextPage();
                    }
                }
            });
            _button2.addItemListener(new ItemListener() {
                public void itemStateChanged(ItemEvent e) {
                    if (e.getStateChange() == ItemEvent.SELECTED) {
                        updateNextPage();
                    }
                }
            });
            JPanel panel = new JPanel(new GridLayout(2, 1));
            panel.add(_button1);
            _button1.setSelected(true);
            panel.add(_button2);
            ButtonGroup group = new ButtonGroup();
            group.add(_button1);
            group.add(_button2);
            addComponent(panel);
            JideSwingUtilities.setOpaqueRecursively(panel, false);
        }

        public void setupWizardButtons() {
            fireButtonEvent(ButtonEvent.ENABLE_BUTTON, ButtonNames.BACK);
            fireButtonEvent(ButtonEvent.ENABLE_BUTTON, ButtonNames.NEXT);
            fireButtonEvent(ButtonEvent.HIDE_BUTTON, ButtonNames.FINISH);
            updateNextPage();
        }

        private void updateNextPage() {
            if (_button1.isSelected() && getOwner() != null) {
                getOwner().setNextPage(getOwner().getPageByTitle("Infos and Warnings"));
            }
            else if (_button2.isSelected() && getOwner() != null) {
                getOwner().setNextPage(getOwner().getPageByTitle("Completing the JIDE Wizard Feature Demo"));
            }
        }
    }

}
